MODULE f90_interfaces
!***********************************************************************
!  FORTRAN 90 module containing interfaces for f90_utilities.
!***********************************************************************
  INTERFACE
     SUBROUTINE swap_i(a,b)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(inout) :: a,b
     END SUBROUTINE swap_i
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_r(a,b)
       USE define_types
       IMPLICIT NONE
       REAL(sp), INTENT(inout) :: a,b
     END SUBROUTINE swap_r
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_d(a,b)
       USE define_types
       IMPLICIT NONE
       REAL(dp), INTENT(inout) :: a,b
     END SUBROUTINE swap_d
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_dv(a,b)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:), INTENT(inout) :: a,b
     END SUBROUTINE swap_dv
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_c(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(spc), INTENT(inout) :: a, b
     END SUBROUTINE swap_c
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_cv(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(spc), DIMENSION(:), INTENT(inout) :: a, b
     END SUBROUTINE swap_cv
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_cm(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(spc), DIMENSION(:,:), INTENT(inout) :: a, b
     END SUBROUTINE swap_cm
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_z(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(dpc), INTENT(inout) :: a, b
     END SUBROUTINE swap_z
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_zv(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(dpc), DIMENSION(:), INTENT(inout) :: a, b
     END SUBROUTINE swap_zv
  END INTERFACE
!
  INTERFACE
     SUBROUTINE swap_zm(a,b)
       USE define_types
       IMPLICIT NONE
       COMPLEX(dpc), DIMENSION(:,:), INTENT(inout) :: a, b
     END SUBROUTINE swap_zm
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_rs(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), INTENT(inout) :: a, b
       LOGICAL(lgt), INTENT(in) :: mask
     END SUBROUTINE masked_swap_rs
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_rv(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:), INTENT(inout) :: a, b
       LOGICAL(lgt), DIMENSION(:), INTENT(in) :: mask
     END SUBROUTINE masked_swap_rv
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_rm(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:,:), INTENT(inout) :: a, b
       LOGICAL(lgt), DIMENSION(:,:), INTENT(in) :: mask
     END SUBROUTINE masked_swap_rm
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_rd(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), INTENT(inout) :: a, b
       LOGICAL(lgt), INTENT(in) :: mask
     END SUBROUTINE masked_swap_rd
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_dv(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:), INTENT(inout) :: a, b
       LOGICAL(lgt), DIMENSION(:), INTENT(in) :: mask
     END SUBROUTINE masked_swap_dv
  END INTERFACE
!
  INTERFACE
     SUBROUTINE masked_swap_dm(a,b,mask)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:,:), INTENT(inout) :: a, b
       LOGICAL(lgt), DIMENSION(:,:), INTENT(in) :: mask
     END SUBROUTINE masked_swap_dm
  END INTERFACE
!
  INTERFACE 
     SUBROUTINE mat_mult_s(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:,:), INTENT(in) :: a, b
       REAL(sp), DIMENSION(:,:), INTENT(out) :: c
       EXTERNAL sgemm
     END SUBROUTINE mat_mult_s
  END INTERFACE
!
  INTERFACE 
     SUBROUTINE mat_mult_d(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:,:), INTENT(in) :: a, b
       REAL(dp), DIMENSION(:,:), INTENT(out) :: c
       EXTERNAL dgemm
     END SUBROUTINE mat_mult_d
  END INTERFACE
!
  INTERFACE 
     SUBROUTINE vv_mult_s(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:), INTENT(in) :: a, b
       REAL(sp), INTENT(out) :: c
       REAL(sp) :: sdot
       EXTERNAL sdot
     END SUBROUTINE vv_mult_s
  END INTERFACE
!
  INTERFACE 
     SUBROUTINE vv_mult_d(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:), INTENT(in) :: a, b
       REAL(dp), INTENT(out) :: c
       REAL(dp) :: ddot
       EXTERNAL ddot
     END SUBROUTINE vv_mult_d
  END INTERFACE
!
  INTERFACE
     SUBROUTINE vm_mult_s(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:), INTENT(in) :: a
       REAL(sp), DIMENSION(:,:), INTENT(in) :: b
       REAL(sp), DIMENSION(:,:), INTENT(out) :: c
       EXTERNAL sgemm
     END SUBROUTINE vm_mult_s
  END INTERFACE
!
  INTERFACE
     SUBROUTINE vm_mult_d(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:), INTENT(in) :: a
       REAL(dp), DIMENSION(:,:), INTENT(in) :: b
       REAL(dp), DIMENSION(:,:), INTENT(out) :: c
       EXTERNAL dgemm
     END SUBROUTINE vm_mult_d
  END INTERFACE
!
  INTERFACE
     SUBROUTINE mv_mult_s(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:,:), INTENT(in) :: a
       REAL(sp), DIMENSION(:), INTENT(in) :: b
       REAL(sp), DIMENSION(:), INTENT(out) :: c
       EXTERNAL sgemv
     END SUBROUTINE mv_mult_s
  END INTERFACE
!
  INTERFACE
     SUBROUTINE mv_mult_d(a,b,c)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:,:), INTENT(in) :: a
       REAL(dp), DIMENSION(:), INTENT(in) :: b
       REAL(dp), DIMENSION(:), INTENT(out) :: c
       EXTERNAL dgemv
     END SUBROUTINE mv_mult_d
  END INTERFACE
!
  INTERFACE
     SUBROUTINE vech_r(a,b,n,ndim,m,mdim,k,option,v)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: n, ndim, m, mdim, k, option
       REAL(sp), DIMENSION(ndim), INTENT(in) :: a
       REAL(sp), DIMENSION(1,mdim), INTENT(in) :: b
       REAL(sp), DIMENSION(k**2), INTENT(inout) :: v
     END SUBROUTINE vech_r
  END INTERFACE
!
  INTERFACE
     SUBROUTINE vech_d(a,b,n,ndim,m,mdim,k,option,v)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: n, ndim, m, mdim, k, option
       REAL(dp), DIMENSION(ndim), INTENT(in) :: a
       REAL(dp), DIMENSION(1,mdim), INTENT(in) :: b
       REAL(dp), DIMENSION(k**2), INTENT(inout) :: v
     END SUBROUTINE vech_d
  END INTERFACE
!
  INTERFACE
     SUBROUTINE cholesky_fac_r(x,u)
       USE define_types
       IMPLICIT NONE
       REAL(sp), DIMENSION(:,:), INTENT(in) :: x
       REAL(sp), DIMENSION(SIZE(x,1),SIZE(x,2)), INTENT(out) :: u
       EXTERNAL spotrf
     END SUBROUTINE cholesky_fac_r
  END INTERFACE
!
  INTERFACE
     SUBROUTINE cholesky_fac_d(x,u)
       USE define_types
       IMPLICIT NONE
       REAL(dp), DIMENSION(:,:), INTENT(in) :: x
       REAL(dp), DIMENSION(SIZE(x,1),SIZE(x,2)), INTENT(out) :: u
       EXTERNAL dpotrf
     END SUBROUTINE cholesky_fac_d
  END INTERFACE
!
  INTERFACE
     FUNCTION assert_eq2(n1,n2,string)
       USE define_types
       IMPLICIT NONE
       CHARACTER(len=*), INTENT(in) :: string
       INTEGER(i4b), INTENT(in) :: n1, n2
       INTEGER(i4b) :: assert_eq2
     END FUNCTION assert_eq2
  END INTERFACE
!
  INTERFACE
     FUNCTION assert_eq3(n1,n2,n3,string)
       USE define_types
       IMPLICIT NONE
       CHARACTER(len=*), INTENT(in) :: string
       INTEGER(i4b), INTENT(in) :: n1, n2, n3
       INTEGER(i4b) :: assert_eq3
     END FUNCTION assert_eq3
  END INTERFACE
!
  INTERFACE
     FUNCTION assert_eq4(n1,n2,n3,n4,string)
       USE define_types
       IMPLICIT NONE
       CHARACTER(len=*), INTENT(in) :: string
       INTEGER(i4b), INTENT(in) :: n1, n2, n3, n4
       INTEGER(i4b) :: assert_eq4
     END FUNCTION assert_eq4
  END INTERFACE
!
  INTERFACE
     FUNCTION assert_eqn(nn,string)
       USE define_types
       IMPLICIT NONE
       CHARACTER(len=*), INTENT(in) :: string
       INTEGER(i4b), DIMENSION(:), INTENT(in) :: nn
       INTEGER(i4b) :: assert_eqn
     END FUNCTION assert_eqn
  END INTERFACE
!
  INTERFACE
     SUBROUTINE error(string)
       USE define_types
       IMPLICIT NONE
       CHARACTER(len=*), INTENT(in) :: string
     END SUBROUTINE error
  END INTERFACE
!
  INTERFACE
     SUBROUTINE i_sort(n,x_in,indices,x_out)
       USE define_types
       IMPLICIT none
       INTEGER(i4b), INTENT(in) :: n
       INTEGER(i4b), INTENT(inout) :: indices
       INTEGER(i4b), DIMENSION(n), INTENT(in) :: x_in
       INTEGER(i4b), DIMENSION(n), INTENT(out) :: x_out
     END SUBROUTINE i_sort
  END INTERFACE
!
  INTERFACE
     SUBROUTINE r_sort(n,x_in,indices,x_out)
       USE define_types
       IMPLICIT none
       INTEGER(i4b), INTENT(in) :: n
       INTEGER(i4b), INTENT(inout) :: indices
       REAL(sp), DIMENSION(n), INTENT(in) :: x_in
       REAL(sp), DIMENSION(n), INTENT(out) :: x_out
     END SUBROUTINE r_sort
  END INTERFACE
!
  INTERFACE
     SUBROUTINE d_sort(n,x_in,indices,x_out)
       USE define_types
       IMPLICIT none
       INTEGER(i4b), INTENT(in) :: n
       INTEGER(i4b), INTENT(inout) :: indices
       REAL(dp), DIMENSION(n), INTENT(in) :: x_in
       REAL(dp), DIMENSION(n), INTENT(out) :: x_out
     END SUBROUTINE d_sort
  END INTERFACE
END MODULE f90_interfaces
