MODULE fcar_interfaces
!***********************************************************************
!  FORTRAN 90 module containing interfaces unique to FCAR subprograms.
!***********************************************************************
  INTERFACE
     SUBROUTINE fit_fcar(max_n,n,k,p,y,x,band_width,u,z,smoother,start,grid,f_dim,f_hat)
       USE define_types
       USE stat_utilities
       USE f90_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, n, k, p, f_dim, smoother, start, grid
       REAL(dp), INTENT(in) :: band_width
       REAL(dp), INTENT(in), DIMENSION(max_n) :: u, z 
       REAL(dp), INTENT(in), DIMENSION(max_n, k) :: y
       REAL(dp), INTENT(in), DIMENSION(max_n, p) :: x
       REAL(dp), INTENT(inout), DIMENSION(max_n, p, k) :: f_hat
       EXTERNAL dgetrf
       EXTERNAL dgetri
     END SUBROUTINE fit_fcar
  END INTERFACE
!
  INTERFACE
     SUBROUTINE fcar_bandwidth(max_n,n,k,p,f_dim,y,x,u,z,smoother,nh,bw_q,ams,bw_vec)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, n, k, p, f_dim, smoother, nh, bw_q
       REAL(dp), INTENT(in), DIMENSION(max_n) :: u, z
       REAL(dp), INTENT(in), DIMENSION(max_n, k) :: y
       REAL(dp), INTENT(in), DIMENSION(max_n, p) :: x
       REAL(dp), INTENT(inout), DIMENSION(nh+1) :: bw_vec, ams
     END SUBROUTINE fcar_bandwidth
  END INTERFACE
!
  INTERFACE
     SUBROUTINE fcar_test(max_n,max_k,n,k,trim,y0_hat,res1,rss0,rss1,rss0_det, &
          rss1_det,test_statistic)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, max_k, n, k
       REAL(dp), INTENT(in) :: trim
       REAL(dp), INTENT(inout) :: rss0_det, rss1_det
       REAL(dp), INTENT(in), DIMENSION(max_n, k) :: y0_hat, res1
       REAL(dp), INTENT(inout) :: test_statistic
       REAL(dp), INTENT(inout), DIMENSION(max_k, k) :: rss0, rss1
     END SUBROUTINE fcar_test
  END INTERFACE
!
  INTERFACE
     SUBROUTINE bootstrap_test(max_n,max_k,n,k,p,trim,y,x,u,z,y0_hat,res1,bw, &
          no_reps,rss0_det,rss1_det,T0,ts_star,p_value)
       USE define_types
       USE f90_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, max_k, n, k, p, no_reps
       REAL(dp), INTENT(in) :: trim, bw, rss0_det, rss1_det, T0
       REAL(dp), INTENT(in), DIMENSION(max_n) :: u, z 
       REAL(dp), INTENT(in), DIMENSION(max_n, p) :: x
       REAL(dp), INTENT(in), DIMENSION(max_n, k) :: y, y0_hat
       REAL(dp), INTENT(inout), DIMENSION(max_n, k) :: res1
       REAL(dp), INTENT(out), DIMENSION(no_reps) :: ts_star
       REAL(dp), INTENT(out) :: p_value
     END SUBROUTINE bootstrap_test
  END INTERFACE
!
  INTERFACE
     SUBROUTINE naive_forecast(n_dim,n,k,p,no_obs,no_cols,m,ts,y,x,u,int_opt,  &
          bw,smoother,d,index,pred,m_dim,pred_error)
       USE define_types
       INTEGER(i4b), INTENT(in) :: n_dim, n, k, p, no_obs, no_cols, m
       INTEGER(i4b), INTENT(in) :: smoother, int_opt, d, index, m_dim
       REAL(dp), INTENT(in) :: bw
       REAL(dp), INTENT(in), DIMENSION(n_dim) :: u
       REAL(dp), INTENT(in), DIMENSION(n_dim, k) :: ts, y
       REAL(dp), INTENT(in), DIMENSION(n_dim, p) :: x
       REAL(dp), INTENT(inout), DIMENSION(n_dim, k) :: pred
       REAL(dp), INTENT(out), DIMENSION(m_dim, k) :: pred_error
     END SUBROUTINE naive_forecast
  END INTERFACE
!
  INTERFACE
     SUBROUTINE multistage_forecast(n_dim,n,k,p,no_obs,f_dim,no_cols,m,ts,y,x,    &
          u,int_opt,bw,smoother,d,index,pred,m_dim,pred_error)
       USE define_types
       INTEGER(i4b), INTENT(in) :: n_dim, n, k, p, no_obs, f_dim, no_cols, m
       INTEGER(i4b), INTENT(in) :: smoother, int_opt, d, index, m_dim
       REAL(dp), INTENT(in) :: bw
       REAL(dp), INTENT(in), DIMENSION(n_dim) :: u
       REAL(dp), INTENT(in), DIMENSION(n_dim, k) :: ts, y
       REAL(dp), INTENT(in), DIMENSION(n_dim, p) :: x
       REAL(dp), INTENT(inout), DIMENSION(n_dim, k) :: pred
       REAL(dp), INTENT(out), DIMENSION(m_dim, k) :: pred_error
     END SUBROUTINE multistage_forecast
  END INTERFACE
!
  INTERFACE
     SUBROUTINE bootstrap_forecast(n_dim,n,k,p,no_obs,no_cols,m,ts,y,x,u,    &
          int_opt,bw,smoother,d,index,b,res,pred,m_dim,pred_error)
       USE define_types
       INTEGER(i4b), INTENT(in) :: n_dim, n, k, p, no_obs, no_cols, m
       INTEGER(i4b), INTENT(in) :: int_opt, smoother, d, index, b, m_dim
       REAL(dp), INTENT(in) :: bw
       REAL(dp), INTENT(in), DIMENSION(n_dim) :: u
       REAL(dp), INTENT(in), DIMENSION(n_dim, no_cols) :: x
       REAL(dp), INTENT(in), DIMENSION(n_dim, k) :: ts, y, res
       REAL(dp), INTENT(inout), DIMENSION(n_dim, k) :: pred
       REAL(dp), INTENT(out), DIMENSION(m_dim, k) :: pred_error
     END SUBROUTINE bootstrap_forecast
  END INTERFACE
!
END MODULE fcar_interfaces
