MODULE nlts_interfaces
!***********************************************************************
!  FORTRAN 90 module containing interfaces for non-linear time series
!  analysis programs.
!***********************************************************************
  INTERFACE
     SUBROUTINE get_mvbilinear_params(max_k,max_p,max_q,file_label,k,p,q,&
		  cp_option,AR_coefs,MA_coefs,cp_dim,cp_coefs)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_k, max_p, max_q, file_label, k
       INTEGER(i4b), INTENT(out) :: p, q, cp_option, cp_dim
       REAL(dp), INTENT(out), DIMENSION(max_k, (max_p*max_k)**2) :: cp_coefs
       REAL(dp), INTENT(out), DIMENSION(max_p, max_k, k) :: AR_coefs
       REAL(dp), INTENT(out), DIMENSION(max_q, max_k, k) :: MA_coefs
     END SUBROUTINE get_mvbilinear_params
  END INTERFACE
!
  INTERFACE
     SUBROUTINE gen_mvbilinear(max_n,max_k,max_p,max_q,n,k,p,q,AR_coefs, &
          MA_coefs,cp_option,cp_dim,cp_coefs,cov_fac_option,cov,extra,x)
       USE define_types
       USE f90_utilities, ONLY : vech
       USE stat_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, max_k, max_p, max_q
       INTEGER(i4b), INTENT(in) :: n, k, p, q, cp_option, cp_dim
       INTEGER(i4b), INTENT(in) :: cov_fac_option, extra
       REAL(dp), INTENT(in), DIMENSION(max_k, k) :: cov
       REAL(dp), INTENT(in), DIMENSION(max_k, cp_dim) :: cp_coefs
       REAL(dp), INTENT(in), DIMENSION(max_p, max_k, k) :: AR_coefs
       REAL(dp), INTENT(in), DIMENSION(max_q, max_k, k) :: MA_coefs
       REAL(dp), INTENT(inout), DIMENSION(max_n, k) :: x
     END SUBROUTINE gen_mvbilinear
  END INTERFACE
!
  INTERFACE
     SUBROUTINE get_mvTAR_params(max_k,max_p,max_q,max_no_regimes,         &
	      file_label,k,p,q,no_regimes,TAR_AR_coefs,TAR_MA_coefs,           &
          AR_lags,MA_lags,thr_resp_ind,thr_lag,thresholds,TAR_covs)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_k, max_p, max_q, max_no_regimes
       INTEGER(i4b), INTENT(in) :: file_label, k
       INTEGER(i4b), INTENT(out) :: p, q, no_regimes, thr_resp_ind, thr_lag
       INTEGER(i4b), INTENT(out), DIMENSION(max_p) :: AR_lags
       INTEGER(i4b), INTENT(out), DIMENSION(max_q) :: MA_lags
       REAL(dp), INTENT(out), DIMENSION(max_no_regimes) :: thresholds
       REAL(dp), INTENT(out), DIMENSION(max_no_regimes, max_k, max_k) :: TAR_covs
       REAL(dp), INTENT(out), DIMENSION(max_no_regimes, max_p, max_k, max_k) :: TAR_AR_coefs
	   REAL(dp), INTENT(out), DIMENSION(max_no_regimes, max_q, max_k, max_k) :: TAR_MA_coefs
     END SUBROUTINE get_mvTAR_params
  END INTERFACE
!
  INTERFACE
     SUBROUTINE gen_mvTAR(max_n,max_k,max_p,max_q,max_no_regimes,          &
          n,k,p,q,no_regimes,TAR_AR_coefs,TAR_MA_coefs,AR_lags,MA_lags,    &
          thr_resp_ind,thr_lag,thresholds,TAR_covs,cov_fac_option,extra,x)
       USE define_types
       USE stat_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, max_k, max_p, max_q, max_no_regimes
       INTEGER(i4b), INTENT(in) :: n, k, p, q, cov_fac_option, extra
       INTEGER(i4b), INTENT(in) :: no_regimes, thr_resp_ind, thr_lag
       INTEGER(i4b), INTENT(in), DIMENSION(p) :: AR_lags
       INTEGER(i4b), INTENT(in), DIMENSION(q) :: MA_lags
       REAL(dp), INTENT(in), DIMENSION(max_no_regimes) :: thresholds
       REAL(dp), INTENT(in), DIMENSION(no_regimes, max_k, k) :: TAR_covs
       REAL(dp), INTENT(in), DIMENSION(no_regimes, max_p, max_k, k) :: TAR_AR_coefs
       REAL(dp), INTENT(in), DIMENSION(no_regimes, max_q, max_k, k) :: TAR_MA_coefs
       REAL(dp), INTENT(out), DIMENSION(max_n, k) :: x
     END SUBROUTINE gen_mvTAR
  END INTERFACE
!
  INTERFACE
     SUBROUTINE get_mvSTAR_params(max_k,max_p,max_no_regimes,file_label,    &
          k,p,sm_func,sm_const,sm_param,STAR_AR_coefs,AR_lags,thr_resp_ind, &
          thr_lag)
       USE define_types
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_k, max_p, max_no_regimes
       INTEGER(i4b), INTENT(in) :: file_label, k
       INTEGER(i4b), INTENT(out) :: p, sm_func, thr_resp_ind, thr_lag
       INTEGER(i4b), INTENT(out), DIMENSION(max_p) :: AR_lags
       REAL(dp), INTENT(out) :: sm_param, sm_const
       REAL(dp), INTENT(out), DIMENSION(max_no_regimes, max_p, max_k, max_k) :: STAR_AR_coefs
     END SUBROUTINE get_mvSTAR_params
  END INTERFACE
!
  INTERFACE
     SUBROUTINE gen_mvSTAR(max_n,max_k,max_p,max_no_regimes,n,k,p,sm_func,   &
          sm_const,sm_param,STAR_AR_coefs,AR_lags,thr_resp_ind,thr_lag,      &
          cov,cov_fac_option,extra,x)
       USE define_types
       USE stat_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_n, max_k, max_p, max_no_regimes
       INTEGER(i4b), INTENT(in) :: n, k, p, extra, cov_fac_option
       INTEGER(i4b), INTENT(in) :: sm_func, thr_resp_ind, thr_lag
       INTEGER(i4b), INTENT(in), DIMENSION(p) :: AR_lags
       REAL(dp), INTENT(in) :: sm_param, sm_const
       REAL(dp), INTENT(in), DIMENSION(max_k, k) :: cov
       REAL(dp), INTENT(in), DIMENSION(max_no_regimes, max_p, max_k, k) :: STAR_AR_coefs
       REAL(dp), INTENT(out), DIMENSION(max_n, k) :: x
     END SUBROUTINE gen_mvSTAR
  END INTERFACE
!
  INTERFACE
     SUBROUTINE get_mvEXPAR_params(max_k,max_p,file_label,k,p,AR_cons,   &
          AR_coefs,expar_coefs,d)
       USE define_types
       USE stat_utilities
       IMPLICIT NONE
       INTEGER(i4b), INTENT(in) :: max_k, max_p, file_label, k
       INTEGER(i4b), INTENT(out) :: p, d
       REAL(dp), INTENT(out), DIMENSION(max_p, max_k, max_k) :: AR_cons, AR_coefs
       REAL(dp), INTENT(out), DIMENSION(max_p, max_k*max_k, max_k) :: expar_coefs
         END SUBROUTINE get_mvEXPAR_params
  END INTERFACE
!
  INTERFACE
    SUBROUTINE gen_mvEXPAR(max_n,max_k,max_p,n,k,p,AR_cons,AR_coefs,    &
         expar_coefs,d,cov,cov_fac_option,extra,x)
      USE define_types
      USE stat_utilities
      IMPLICIT NONE
      INTEGER(i4b), INTENT(in) :: max_n, max_k, max_p, n, k, p, d
      INTEGER(i4b), INTENT(in) :: cov_fac_option, extra
      REAL(dp), INTENT(in), DIMENSION(max_k, k) :: cov
      REAL(dp), INTENT(in), DIMENSION(max_p, max_k, k) :: AR_cons, AR_coefs
      REAL(dp), INTENT(in), DIMENSION(max_p, max_k*max_k, max_k) :: expar_coefs
      REAL(dp), INTENT(out), DIMENSION(max_n, k) :: x
    END SUBROUTINE gen_mvEXPAR
  END INTERFACE
!
END MODULE nlts_interfaces
