% MATLAB code
% File: VFCAR-M.m
% 
% Matlab code for bivariate VFCAR estimation, with LR type test statistic (12.31)
%
% Reference:
% Harvill, J.L. and Ray, B.K. (2006).
%   Functional coefficient autoregressive models for vector time series.
%   Computational Statistics & Data Analysis, 50, 3547-3566.
%   DOI: 10.1016/j.csda.2005.07.016.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% clear all;
 
no_ns = 1;             % no of sample sizes
extra = 0;             % extra observations
k     = 2;             % dimension of time series
max_k = 2;             % Largest allowable number of components in the vector.
max_n = 600;           % Largest allowable sample size ((n + extra).LE. max_n).
                       % Also serves as maximum number of bootstrap replications.
nh    = 50;            % Number of bandwidths to consider in opt. bandwidth selection.
bw_q  = 4;             % Number of subsections of data in bandwidth selection.

no_corrs = 2;          % number of cross-correlations
sds      = 1;          % standard deviations are 1
means    = 0;          % mean is zero
intercept_option = 1;  % intercept option
plusone = 1; 
trim    = 10;          % percentage of FCAR estimates to trim for inference
                       % the trimming percentage must be between 0 and 20
alpha   = 5;           % nominal level
model_option = 0;      % model

AR_fit_order = 1;      % FCAR order 
p     = 1;
max_p = 1;             % Largest allowable AR order in all models (p.LE max_p)
                       % max_q: Largest allowable MA order in all models (q .LE max_q)
n     = 66;            % Length time series for file: treering.dat         

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mu    = [0 0];
% SIGMA = [1 0.5; 0.5 1];
% x     = mvnrnd(mu,SIGMA,n);    %  input time series     

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Functional variable loop (response index and delay) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(intercept_option == 1) 
  xr(1:n,1) = 1.0;
end

no_cols = k*AR_fit_order + plusone;

for func_resp_ind = 1:k            % start 1st big loop
  band_width = 0.0;
  for func_delay = 1:max(p,1)      % start 2nd big loop
    max_lag         = max(AR_fit_order, func_delay);
    no_obs          = n - 1;    % max_lag;
    y(1:no_obs,1:k) = x((max_lag+1):n,1:k);                 
    for i = 1:AR_fit_order
      xr(1:no_obs,(k*(i-1)+(1+plusone)):(i*k+plusone)) = x((max_lag-i+1):(n-i),1:k);
    end
         
    beg_sub = 1 + max_lag - func_delay;
    end_sub = no_obs + max_lag - func_delay;
             
    func_var(1:no_obs) = x(beg_sub:end_sub,func_resp_ind);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Compute fitted values from linear regression 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    beta_dim = max_k*max_p+1;
    xrr      = xr(1:no_obs,1:beta_dim);
     
    [beta_hat,se_b,mse] = lscov(xrr,y);     
    y0_hat              = xrr * beta_hat;
    u                   = func_var;
    z                   = func_var;
             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Compute bandwidth
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    dn    = no_obs;
    m     = floor(0.1*dn);
    ams   = zeros(nh+1,1);
    f_dim = max_p*max_k;
            
    range_of_u = range(u);   % maxu - minu;
    a          = no_obs^(-0.20);
    step       = range_of_u*a*1.5/(nh);
    bw_vec(1)  = 0.5*range_of_u*a;
    for i = 2:nh+1
       bw_vec(i) = bw_vec(i-1) + step;  % 0.05;
    end
            
    for i = 1: bw_q
      n1 = no_obs - i*m;
      for j = 1:nh+1
        h = bw_vec(j) * (dn/(n1))^0.2;
        
%%%%    start  compute fit_fcar, within bandwidth %%%%%%%
%%%%
        band_width1 = h;
        p2 = 2*no_cols;
        s0 = 0.0;
        s1 = 0.0;
        for i = 1:p2
          s0(i,i) = 1.0;
         end
         dxnew(1:n1, 1:no_cols) = xr(1:n1, 1:no_cols);             
        

        for i = n1+1:n1+m  % start loop i
          for t = 1:n1
            w0epa(t) = (u(t)-z(i))/band_width1; 
            
%%%         start Epanechnikov smoother 
%%%
            if(abs(w0epa(t))<= 1) 
              w0epa(t) = 0.75*(1.0-w0epa(t)*w0epa(t));
            else 
              w0epa(t) = 0;
            end
%%%         end Epanechnikov smoother
                
            w0(t) = w0epa(t) / band_width1;
                        
            work_space(t,1:no_cols)      = w0(t) * dxnew(t,1:no_cols);                        
            dxnew(t,(no_cols+1):p2)      = (u(t) - z(i))*xr(t,1:no_cols);   
            work_space(t,(no_cols+1):p2) = w0(t)*dxnew(t,(no_cols+1):p2);
          end    % end loop t = 1:n1
              
          temp = dxnew'*work_space;  
          s0   = (1.0/(n1*n1))*s0 + temp;  
          s02  = inv(s0);   
          for t = 1:n1
            wy(t,1:k) = w0(t)* y(t,1:k);
           end
           s1 = dxnew'*wy;   
           s2 = s02*s1;   
 
           ss2 = (s0\s1);               %% CHECK 
           f_hat(i,1:no_cols,1:k) = ss2(1:no_cols,1:k);                    
       end    % end loop i = n1+1:n1+m
           
%%%%%%%  end compute fit_fcar   %%%%%%%%%%%%%%%%%%%%

       for t = n1+1:n1+m % n1+m
         xxdat(t,1:no_cols)     = xr(t,1:no_cols);   
         ffdat(t,1:no_cols,1:k) = f_hat(t,1:no_cols,1:k);
         y_hat                  = xxdat(t,:)*[ffdat(t,:,1); ffdat(t,:,2)]';              
         err(1:k)               = y(t,1:k) - y_hat(1:k);
         for jj = 1:k
           ams(j) = ams(j) + err(jj)*err(jj);
         end  % end loop jj = 1:k
       end    % end t = n1+1:n1+m

     
     end      % end loop j = 1:nh+1
end           % end loop i = 1:bw_q
  
%%%%% end computing bandwidth %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  end  % end func_delay loop, 2nd big loop
end    % end func_resp_ind loop, 1st big loop
  
 B,ind] = sort(ams);
opt_bw  = bw_vec(ind(1));

%%%%%%%%%  start  compute fit_fcar with optimal bandwidth %%%%%%%%%
   
p2 = 2*no_cols;
s0 = 0.0;
s1 = 0.0;
for i = 1:p2
  s0(i,i) = 1.0;
end

band_width               = opt_bw;
dx1(1:no_obs, 1:no_cols) = xr(1:no_obs, 1:no_cols); 
for i = 1:no_obs
  for t = 1:no_obs
    w01(t) = (u(t)-z(i))/band_width;  
    
%%  start Epanechnikov smoother 
%%
    if(abs(w01(t))<= 1) 
      w01(t) = 0.75*(1.0-w01(t)*w01(t));
    else 
      w01(t) = 0;
    end
%%  end Epanechnikov smoother
                   
    w01(t)                        = w01(t)/band_width;
    work_space1(t,1:no_cols)      = w01(t) * dx1(t,1:no_cols);         
    dx1(t,(no_cols+1):p2)         = (u(t) - z(i))*xr(t,1:no_cols);   
    work_space1(t,(no_cols+1):p2) = w01(t)*dx1(t,(no_cols+1):p2);
  end    % end loop t = 1:n1
               
  temp1  = dx1'*work_space1;
  s0new  = (1.0/(no_obs*no_obs))*s0 + temp1;
  s02new = inv(s0new);    
  for t = 1:no_obs
    wy(t,1:k) = w01(t)* y(t,1:k);
  end
  s1new                     = dx1'*wy;
  s2new                     = s02new*s1new;
  f_hatnew(i,1:no_cols,1:k) = s2new(1:no_cols,1:k);                    
end       % end loop i=1:no_obs 
    
     
%%%%%%%  end compute fit_fcar   %%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    start compute fitted values from FCAR model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

y1_hat(1:no_obs,1:k) = 0.0;
z1_hat(1:no_obs,1:k) = 0.0;
for j = 1:no_obs
  for l = 1:no_cols
    z1_hat(j,1:k) = xrr(j,l)*f_hatnew(j,l,1:k);
    y1_hat(j,1:k) = y1_hat(j,1:k)+z1_hat(j,1:k);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   Sort values according to the ordered values of the functional
%%   variable.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i = 1:no_obs
  u_index(i) = i;
end
z(1:no_obs)  = u(1:no_obs);
[Bs,u_index] = sort(u);
for i = 1:no_obs
  index_of_u        = u_index(i);
  y_s(i,1:k)        = y(index_of_u,1:k);
  y0_hat_s(i,1:k)   = y0_hat(index_of_u,1:k);
  y1_hat_s(i,1:k)   = y1_hat(index_of_u,1:k);
  xr_s(i,1:no_cols) = xr(index_of_u,1:no_cols);
end

y0_hat_s(1:no_obs,1:k) = y_s(1:no_obs,1:k) - y0_hat_s(1:no_obs,1:k);
y1_hat_s(1:no_obs,1:k) = y_s(1:no_obs,1:k) - y1_hat_s(1:no_obs,1:k);
T0 = 0.0;
     
if (trim > 0.0) 
    perc      = (100.0 - (trim))/100.0;
    upper_lim = round(perc*no_obs);
    perc      = 1.0 - perc;
    lower_lim = round(perc*no_obs + 1);
    nn        = upper_lim - lower_lim;
    for i = 1: nn
      index         = lower_lim + i - 1;
      ures0(i, 1:k) = y0_hat_s(index, 1:k);
      ures1(i, 1:k) = y1_hat_s(index, 1:k);
     end
   else
      nn        = no_obs;
      lower_lim = 1;
      upper_lim = no_obs;
      ures0(1:no_obs, 1:k) = y0_hat_s(1:no_obs, 1:k);
      ures1(1:no_obs, 1:k) = y1_hat_s(1:no_obs, 1:k);
end
     
rss0_det = 0.0; 
rss1_det = 0.0;
if (k == 1)
    for t = 1:nn
       rss0_det = rss0_det + ures0(t,1)*ures0(t,1);
       rss1_det = rss1_det + ures1(t,1)*ures1(t,1);
     end
     
     else
     
       rss0(1:k,1:k) = ures0(1:nn,1:k)'*ures0(1:nn,1:k);
       rss1(1:k,1:k) = ures1(1:nn,1:k)'*ures1(1:nn,1:k);
       for dim = 1:k
         rss0_det = rss0_det + rss0(dim,dim);
         rss1_det = rss1_det + rss1(dim,dim);
        end
end

lambda         = sqrt(rss1_det/rss0_det);
test_statistic = (1.0 - sqrt(lambda))/sqrt(lambda); 

%%%%%% end computing fcar_test  %%%%%%%%%%%

%%  split f_hatnew in separate files  %%%%

fhatnew11 = f_hatnew(:,1,1);
fhatnew12 = f_hatnew(:,2,1);
fhatnew13 = f_hatnew(:,3,1);

fhatnew21 = f_hatnew(:,1,2);
fhatnew22 = f_hatnew(:,2,2);
fhatnew23 = f_hatnew(:,3,2);

